;-----------------------------------------------------------------------------
; LCD.INC LCD driver in 4-bit nibble mode and use of busy flag
;
;declare in main:
;
;	Cblock	0x0c
;	LCDtemp
;	LCDbuf
;
;-----------------------------------------------------------------------------
;
#define	RS	LCDport,1
#define RW	LCDport,2
#define EN	LCDport,3
;
;-----------------------------------------------------------------------------
;***** Macros *****

LCDinit macro
        call    LCD_init	; LCD initialization       
        endm

LCDchar macro   LCDarg		; write out the character on LCD
        movlw   LCDarg
        call    LCDdata
        endm

LCDw    macro
        call    LCDdata
        endm

LCDcmd  macro   LCDcommand     ; send the command to LCD
        movlw   LCDcommand
        call    LCDcomd
        endm

LCDline macro   line_num
        IF (line_num == 1)
                LCDcmd  LCDL1  ; Start macro with "First Line" instruction
        ELSE
             IF (line_num == 2)
                LCDcmd  LCDL2  ; Start macro with "Second Line" instruction
             ELSE
             ENDIF
        ENDIF
        endm

LCD_DDAdr macro DDRamAddress
        Local   value = DDRamAddress | b'10000000'      ; Beginning of DDRAM 
        IF (DDRamAddress > 0x67)
                ERROR "Wrong DDRAM address in LCD_DDAdr"
        ELSE
                movlw   value
                call    LCDcomd
        ENDIF
        endm

LCD_CGAdr macro CGRamAddress
        Local   value = CGRamAddress | b'01000000'      ; Beginning of CGRAM
        IF (CGRamAddress > b'00111111')
                ERROR "Wrong CGRAM address in LCD_CGAdr"
        ELSE
                movlw   value
                call    LCDcomd
        ENDIF
        endm

;-----------------------------------------------------------------------------
;***** LCD commands *****

        CONSTANT  LCDEM8 = b'00110000'  ; 8 Bit Mode, 2 Lines
        CONSTANT  LCDDZ  = b'10000000'  ; Write 0 u DDRAM
        CONSTANT  LCDEM4 = b'00100000'  ; 4 Bit Mode, 2 Lines

;***** Standard commands for LCD initializatoion *****

        CONSTANT  LCD2L   = b'00101000' ; Function: 4 bit 2 lines 
        CONSTANT  LCDCONT = b'00001100' ; Display control: Display ON,
                                        ; Cursor OFF, blink OFF
        CONSTANT  LCDSH   = b'00101000' ; Display mode: AutoInc cursor
					; ,NoDisplayAutoShift

;***** Standard LCD commands *****

; In order to send one of these commands to LCD, we need to use LCDcmd
; macro, ex. "LCDcmd LCDCLR"

        CONSTANT  LCDCLR  = b'00000001' ; clears display, resets cursor
        CONSTANT  LCDCH   = b'00000010' ; cursor home
        CONSTANT  LCDCR   = b'00000110' ; cursor move direction right
        CONSTANT  LCDCL   = b'00000100' ; cursor move direction left
        CONSTANT  LCDSL   = b'00011000' ; shifts display content left
        CONSTANT  LCDSR   = b'00011100' ; shifts display content right
        CONSTANT  LCDL1   = b'10000000' ; selects line 1
        CONSTANT  LCDL2   = b'11000000' ; selects line 2


;-----------------------------------------------------------------------------
;Delay: 
;This subroutine is required for other routines.
;It is an efficient, imprecise time delay routine. 

LCDDelay      movwf       LCDtemp           ;Efficient,imprecise delay routine 
              addlw       1 
              btfsc       STATUS, Z 
              incfsz      LCDtemp, f 
              goto        $ - 3 
              return 

;-----------------------------------------------------------------------------
;LCD_Nbl: 
;This routine outputs the upper four bits of the "w" register to the upper
;four bits of the LCD module.  RS must be set/reset before calling this 
;subroutine.I used the four "bcf" instructions to clear the upper four
;bits of the RS rather something more efficient in order to keep this routine
;compatible with the rest of your program (and to save RAM.) 

LCD_Nbl
              bsf         EN                  ;Clock out RS and RW.
              iorlw       0x0F                ;Write the upper nybble in W to 
              andwf       PORTB, f            ;the upper four bits in PORTB.
              andlw       0xF0
              iorwf       PORTB, f
              bcf         EN                  ;Clock out the four data bits. 
              return                          ;Done--Return.  

;-----------------------------------------------------------------------------
;LCD_BF: 
;This routine reads the contents of the HD44780's address counter 
;(and busy flag) and puts the result in the PIC's w register. The busy flag
;is put in the most significant bit of the w register and the address is 
;bits 0-6. 

LCD_BF
              BANK1 
              movlw       0xF1                ;Set PORTB.4-7 to input.
              movwf       LCDtris
              BANK0  
              bcf         RS                  ;Select the instruction register.
              bsf         RW                  ;This is a READ command. 
              bsf         EN                  ;Clock out the RS and RW bits. 
              movf        PORTB, w            ;Get the nybble
              bcf         EN
              andlw       0xf0                ;Clean up the nybble
              movwf       LCDtemp 
              bsf         EN                  ;Clock in the lower nybble
              swapf       PORTB, w            ;Get lower nybble
              bcf         EN 
              andlw       0x0f                ;Clean up the nybble
              iorwf       LCDtemp, f          ;put together and store in LCDtemp
              return                          ;Done -- return. 

;-----------------------------------------------------------------------------
;LCD_Cmd: 
;This routine writes the contents of the PIC's w register to the HD44780's 
;instruction register.  Use this routine by storing a comand in the w register 
;and call this routine.The command will then be written to the LCD module. 

LCDcomd       movwf       LCDbuf              ;Store w
LCD_CmdLoop   call        LCD_BF              ;Get the busy flag
              btfsc       LCDtemp,7           ;is in bit 7
              goto        LCD_CmdLoop         ;  Yes.  Re-test busy flag.
              BANK1                           ;Make PORTB.1-7 Outputs
              movlw       0x01
              movwf       LCDtris
              BANK0
              bcf         RW                  ;This is a WRITE command.
              bcf         RS                  ;This is a COMMAND. 
              movf        LCDbuf, w            ;Restore to-be-sent byte. 
              call        LCD_Nbl             ;Output the upper nybble. 
              swapf       LCDbuf, w  
              call        LCD_Nbl             ;Output the lower nybble. 
              return  

;-----------------------------------------------------------------------------
;LCD_Txt: 
;This routine writes the contents of the PIC's w register to the HD44780's 
;data register.To use this routine, put the text you want to be displayed 
;(or CGRAM data) in w then call this subroutine.  The contents of w will 
;then be written to the LCD's DDRAM (or CGRAM) at the current address. 

LCDdata       movwf       LCDbuf              ;Store w
LCD_TxtLoop   call        LCD_BF              ;Get the busy flag
              btfsc       LCDtemp,7
              goto        LCD_TxtLoop         ;  yes.  Re-test busy flag.
              BANK1                           ;Make PORTB.1-7 Outputs
              movlw       0x01
              movwf       LCDtris
              BANK0
              bcf         RW                  ;This is a WRITE command.
              bsf         RS                  ;Select the data register.
              movf        LCDbuf, w           ;Restore to-be-sent byte. 
              call        LCD_Nbl             ;Output the upper nybble. 
              swapf       LCDbuf, w  
              call        LCD_Nbl             ;Output the lower nybble. 
              return  

;-----------------------------------------------------------------------------
;LCD_Init: 
;This subroutine initializes the LCD module for operation.  It takes about 
;0.33 seconds to execute. 

LCD_init      
        BANK1
        clrf        OPTION_REG
        movlw       b'00000001'
        movwf       LCDtris
        BANK0
        clrf        LCDport
        bcf         RS                  ;The following will write to the IR.
        movlw       0xe0                ;Wait at least 15 milliseconds.
        call        LCDDelay  
        movlw       b'00110000'         ;Function set 
        call        LCD_Nbl 
        movlw       0xf6                ;Wait at least 4.1 milliseconds.
        call        LCDDelay 
        movlw       b'00110000'         ;Function set 
        call        LCD_Nbl 
        movlw       0xfE                ;Wait at least 100 microseconds.
        call        LCDDelay 
        movlw       b'00110000'         ;Function set 
        call        LCD_Nbl 
        movlw       0xFE                ;Wait at least 100 microseconds. 
        call        LCDDelay 
        movlw       b'00100000'         ;Select 4-bit interface - 0 0 1 0 0 0 0 0 
        call        LCDcomd
        movlw       b'00101000'         ;Function Set -- 0 0 1 DL N F X X
        call        LCDcomd
        movlw       b'00001100'         ;Entry Mode Set -- 0 0 0 0 0 1 ID S
        call        LCDcomd
        movlw       b'00000001'         ;Display On/Off Control - 0 0 0 0 1 D C B
        call        LCDcomd
        return 

;-----------------------------------------------------------------------------
;LCDmove: Moves the cursor to the address specified in w. 

LCDmove         macro   Address 
                movlw   Address | 0x80 
                call    LCD_Cmd 
                endm 



;-----------------------------------------------------------------------------

;LCDclear: Clears the display and moves cursor to address 0x00. 
LCDclear        macro 
                movlw   0x01 
                call    LCD_Cmd 
                endm 

